package mos.moduyun.com.sdk.yun;

import org.apache.logging.log4j.util.Base64Util;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

public class Client {
    private String accesskey; //accesskey
    private String secretkey; //secretkey
    private String charset="utf-8"; //字符编码
    private int connectTimeout=300000; //连接超时
    private int socketTimeout=300000; //socket超时

    private String url="https://mpi.moduyun.com"; //上传地址
    private Long fileSize=62914560L; //当文件超个大小,自动使用分片上传(最大值60M)
    private Integer partSize=62914560; //分片上传的文件平均大小

    public Client(){}

    public Client(String accesskey, String secretkey) {
        this.accesskey = accesskey;
        this.secretkey = secretkey;
    }

    public Client(String accesskey, String secretkey, String charset, int connectTimeout, int socketTimeout) {
        this.accesskey = accesskey;
        this.secretkey = secretkey;
        this.charset = charset;
        this.connectTimeout = connectTimeout;
        this.socketTimeout = socketTimeout;
    }

    //创建文件夹
    /**
     * @param region 地域
     * @param bucketName bucket名称
     * @param fileName 文件夹的完整路径
     * @return
     */
    public String createFolder(String region,String bucketName,String fileName,Map<String,String> parameter){
            String url=this.url+"/bucket/createFolder";
            Map<String,String>  map=new HashMap<>();
            String timeRub = System.currentTimeMillis()+"";
            String sig= ClientUtil.MD5(((this.secretkey+timeRub)));
            map.put("accessKeyId",this.accesskey);
            map.put("sig",sig);
            map.put("timeRub",timeRub);
            map.put("region",region);
            map.put("bucketName",bucketName);
            map.put("fileName",fileName);
            if(parameter!=null){
                map.put("callbackUrl",parameter.get("callbackUrl"));
                map.put("callbackParameter",parameter.get("callbackParameter"));
            }
            String msg = ClientUtil.postFile(null, url, map,this.charset, this.connectTimeout,this.socketTimeout);
            return msg;
    }




    //上传文件
    public String  uploadFile(File file,String region,String bucketName,String fileName,Map<String,String> parameter){
        String url=this.url+"/bucket/simpleUploadFile";
        Map<String,String>  map=new HashMap<>();
        String timeRub = System.currentTimeMillis()+"";
        String sig= ClientUtil.MD5((this.secretkey+timeRub));
        map.put("accessKeyId",this.accesskey);
        map.put("sig",sig);
        map.put("timeRub",timeRub);
        map.put("region",region);
        map.put("bucketName",bucketName);
        map.put("fileName",fileName);
        if(parameter!=null){
            map.put("storageClass",parameter.get("storageClass"));
            map.put("jurisdiction",parameter.get("jurisdiction"));
            map.put("sseAlgorithm",parameter.get("sseAlgorithm"));
            map.put("metaData",parameter.get("metaData"));
            map.put("doLoadName",parameter.get("doLoadName"));
            map.put("code",parameter.get("code"));
            map.put("expires",parameter.get("expires"));
            map.put("callbackUrl",parameter.get("callbackUrl"));
        }
        String msg="";
        try {
            InputStream inputStream=new FileInputStream(file);
            String checkParameter = checkParameter(map);
            if(checkParameter.indexOf("200")==-1){
                inputStream.close();
                return checkParameter;
            }
            if(file.length()<=this.fileSize){
                map.put("callbackParameter",parameter.get("callbackParameter"));
                //简单上传
                msg = ClientUtil.postFile(inputStream, url, map, charset, connectTimeout, socketTimeout);
            }else{
                    //分片上传
                    String uploadId = ClientUtil.postFile(null, this.url + "/bucket/getUploadId", map, charset, connectTimeout, socketTimeout);
                    map.put("uploadId",uploadId);
                    int partCount  =(int)(file.length() / this.partSize);
                    int mun = (int) file.length() % this.partSize;
                    if (mun != 0) {
                        partCount++;
                    }
                    byte[] bytes=new byte[this.partSize];
                    int count=1;
                    while (inputStream.read(bytes,0, bytes.length)!=-1){
                        InputStream sbs = new ByteArrayInputStream(bytes);
                        if(partCount==count){
                            map.put("callbackParameter",parameter.get("callbackParameter"));
                        }
                        if(mun != 0&&count==(partCount-1)){
                            bytes=new byte[mun];
                        }
                        System.out.println("正在上传分片:"+count);
                        map.put("partNumber",count+"");
                        map.put("partSize",sbs.available()+"");
                        map.put("partCount",partCount+"");
                        msg= ClientUtil.postFile(sbs,url,map,this.charset,this.connectTimeout,this.socketTimeout);
                        sbs.close();
                        count++;
                    }
            }
        }catch (IOException e){
            e.printStackTrace();
        }
        return msg;
    }



    //参数校验
    public String checkParameter(Map<String,String> parameter){
        String url=this.url+"/bucket/checkParameter";
        String msg = ClientUtil.postFile(null, url, parameter,this.charset, this.connectTimeout,this.socketTimeout);
        return msg;
    }


    //获取公共文件下载链接
    public String getPublicFileLink(String region,String bucketName,String fileName,String versionId){
        StringBuffer stringBuffer=new StringBuffer();
        stringBuffer.append(this.url);
        stringBuffer.append("/bucket/downLoadFile");
        stringBuffer.append("?region="+region);
        stringBuffer.append("&bucketName="+bucketName);
        stringBuffer.append("&fileName="+fileName);
        stringBuffer.append("&versionId="+versionId);
        return stringBuffer.toString();
    }

    //获取私有文件下载链接
    public String getPrivateFileLink(String region,String bucketName,String fileName,String versionId){
        String link = this.getPublicFileLink(region, bucketName, fileName,versionId);
        StringBuffer stringBuffer=new StringBuffer();
        stringBuffer.append(link);
        stringBuffer.append("&accessKeyId="+Base64Util.encode(this.accesskey));
        long currentTimeMillis = System.currentTimeMillis();
        stringBuffer.append("&timeRub="+currentTimeMillis);
        stringBuffer.append("&sig="+ ClientUtil.MD5(this.secretkey + currentTimeMillis));
        return stringBuffer.toString();
    }

}
